@@ -93,8 +93,9 @@ G_DEFINE_TYPE (GdlSwitcher, gdl_switcher, GTK_TYPE_NOTEBOOK)
             GDL_SWITCHER_STYLE_TOOLBAR ? switcher->priv->toolbar_style : \
             switcher->priv->switcher_style)
 
-#define H_PADDING 2
-#define V_PADDING 2
+#define H_PADDING 0
+#define V_PADDING 0
+#define V_SPACING 2
 
 /* Utility functions.  */
 
@@ -252,10 +253,9 @@ button_toggled_callback (GtkToggleButton *toggle_button,
 
 /* Returns -1 if layout didn't happen because a resize request was queued */
 static int
-layout_buttons (GdlSwitcher *switcher)
+layout_buttons (GdlSwitcher *switcher, GtkAllocation* allocation)
 {
-    GtkRequisition client_requisition = {0,};
-    GtkAllocation allocation;
+    gint min_height, nat_height;
     GdlSwitcherStyle switcher_style;
     gboolean icons_only;
     int num_btns = g_slist_length (switcher->priv->buttons);
@@ -270,14 +270,12 @@ layout_buttons (GdlSwitcher *switcher)
     int i;
     int rows_count;
     int last_buttons_height;
-
-    gtk_widget_get_allocation (GTK_WIDGET (switcher), &allocation);
     
     last_buttons_height = switcher->priv->buttons_height_request;
     
-    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->size_request (GTK_WIDGET (switcher), &client_requisition);
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->get_preferred_height (GTK_WIDGET (switcher), &min_height, &nat_height);
 
-    y = allocation.y + allocation.height - V_PADDING - 1;
+    y = allocation->y + allocation->height - V_PADDING - 1;
 
     if (num_btns == 0)
         return y;
@@ -291,18 +289,18 @@ layout_buttons (GdlSwitcher *switcher)
         GtkRequisition requisition;
 
         button = p->data;
-        gtk_widget_size_request (GTK_WIDGET (button->button_widget),
-                                 &requisition);
+        gtk_widget_get_preferred_size (GTK_WIDGET (button->button_widget),
+                                       &requisition, NULL);
         optimal_layout_width += requisition.width + H_PADDING;
         max_btn_height = MAX (max_btn_height, requisition.height);
         max_btn_width = MAX (max_btn_width, requisition.width);    
     }
 
     /* Figure out how many rows and columns we'll use. */
-    btns_per_row = allocation.width / (max_btn_width + H_PADDING);
+    btns_per_row = allocation->width / (max_btn_width + H_PADDING);
     
     /* If all the buttons could fit in the single row, have it so */
-    if (allocation.width >= optimal_layout_width)
+    if (allocation->width >= optimal_layout_width)
     {
         btns_per_row = num_btns;
     }
@@ -379,7 +377,7 @@ layout_buttons (GdlSwitcher *switcher)
         /* Check for possible size over flow (taking into account client
          * requisition
          */
-        if (y < (allocation.y + client_requisition.height)) {
+        if (y < (allocation->y + min_height)) {
             /* We have an overflow: Insufficient allocation */
             if (last_buttons_height < switcher->priv->buttons_height_request) {
                 /* Request for a new resize */
@@ -387,24 +385,26 @@ layout_buttons (GdlSwitcher *switcher)
                 return -1;
             }
         }
-        x = H_PADDING + allocation.x;
+        x = H_PADDING + allocation->x;
         len = g_slist_length (rows[i]);
         if (switcher_style == GDL_SWITCHER_STYLE_TEXT ||
             switcher_style == GDL_SWITCHER_STYLE_BOTH)
-            extra_width = (allocation.width - (len * max_btn_width )
+            extra_width = (allocation->width - (len * max_btn_width )
                            - (len * H_PADDING)) / len;
         else
             extra_width = 0;
         for (p = rows [i]; p != NULL; p = p->next) {
             GtkAllocation child_allocation;
-            
+            GtkStyleContext *style_context;
+            GtkJunctionSides junction = 0;
+
             child_allocation.x = x;
             child_allocation.y = y;
             if (rows_count == 1 && row_number == 0)
             {
                 GtkRequisition child_requisition;
-                gtk_widget_size_request (GTK_WIDGET (p->data),
-                                         &child_requisition);
+                gtk_widget_get_preferred_size (GTK_WIDGET (p->data),
+                                               &child_requisition, NULL);
                 child_allocation.width = child_requisition.width;
             }
             else
@@ -413,14 +413,35 @@ layout_buttons (GdlSwitcher *switcher)
             }
             child_allocation.height = max_btn_height;
 
+            style_context = gtk_widget_get_style_context (GTK_WIDGET (p->data));
+
+            if (row_last) {
+                if (i) {
+                    junction |= GTK_JUNCTION_TOP;
+                }
+                if (i != row_last) {
+                    junction |= GTK_JUNCTION_BOTTOM;
+                }
+            }
+
+            if (p->next) {
+                junction |= GTK_JUNCTION_RIGHT;
+            }
+
+            if (p != rows [i]) {
+                junction |= GTK_JUNCTION_LEFT;
+            }
+
+            gtk_style_context_set_junction_sides (style_context, junction);
+
             gtk_widget_size_allocate (GTK_WIDGET (p->data), &child_allocation);
 
             x += child_allocation.width + H_PADDING;
         }
-
-        y -= V_PADDING;
     }
-    
+
+    y -= V_SPACING;
+
     for (i = 0; i <= row_last; i ++)
         g_slist_free (rows [i]);
     g_free (rows);
@@ -429,27 +450,25 @@ layout_buttons (GdlSwitcher *switcher)
 }
 
 static void
-do_layout (GdlSwitcher *switcher)
+do_layout (GdlSwitcher *switcher, GtkAllocation* allocation)
 {
-    GtkAllocation allocation;
+    
     GtkAllocation child_allocation;
     int y;
-
-    gtk_widget_get_allocation (GTK_WIDGET (switcher), &allocation);
     
     if (switcher->priv->show) {
-        y = layout_buttons (switcher);
+        y = layout_buttons (switcher, allocation);
         if (y < 0) /* Layout did not happen and a resize was requested */
             return;
     }
     else
-        y = allocation.y + allocation.height;
+        y = allocation->y + allocation->height;
     
     /* Place the parent widget.  */
-    child_allocation.x = allocation.x;
-    child_allocation.y = allocation.y;
-    child_allocation.width = allocation.width;
-    child_allocation.height = y - allocation.y;
+    child_allocation.x = allocation->x;
+    child_allocation.y = allocation->y;
+    child_allocation.width = allocation->width;
+    child_allocation.height = y - allocation->y;
     
     GTK_WIDGET_CLASS (gdl_switcher_parent_class)->size_allocate (GTK_WIDGET (switcher), &child_allocation);
 }
@@ -502,13 +521,13 @@ gdl_switcher_remove (GtkContainer *container, GtkWidget *widget)
 /* GtkWidget methods.  */
 
 static void
-gdl_switcher_size_request (GtkWidget *widget, GtkRequisition *requisition)
+gdl_switcher_get_preferred_width (GtkWidget *widget, gint *minimum, gint *natural)
 {
     GdlSwitcher *switcher = GDL_SWITCHER (widget);
     GSList *p;
     gint button_height = 0;
     
-    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->size_request (GTK_WIDGET (switcher), requisition);
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->get_preferred_width (GTK_WIDGET (switcher), minimum, natural);
 
     if (!switcher->priv->show)
         return;
@@ -516,42 +535,65 @@ gdl_switcher_size_request (GtkWidget *widget, GtkRequisition *requisition)
     for (p = switcher->priv->buttons; p != NULL; p = p->next) {
         gint button_width;
         Button *button = p->data;
-        GtkRequisition button_requisition;
+        gint min, nat;
 
-        gtk_widget_size_request (button->button_widget, &button_requisition);
-        button_width = button_requisition.width + 2 * H_PADDING;
-        requisition->width = MAX (requisition->width, button_width);
-        button_height = MAX (button_height,
-                             button_requisition.height + 2 * V_PADDING);
+        gtk_widget_get_preferred_width(button->button_widget, &min, &nat);
+        *minimum = MAX (*minimum, min + 2 * H_PADDING);
+        *natural = MAX (*natural, nat + 2 * H_PADDING);
     }
+}
+
+static void
+gdl_switcher_get_preferred_height (GtkWidget *widget, gint *minimum, gint *natural)
+{
+    GdlSwitcher *switcher = GDL_SWITCHER (widget);
+    GSList *p;
+    gint button_min = 0;
+    gint button_nat = 0;
+ 
+    GTK_WIDGET_CLASS (gdl_switcher_parent_class)->get_preferred_height (GTK_WIDGET (switcher), minimum, natural);
+
+    if (!switcher->priv->show)
+        return;
     
+    for (p = switcher->priv->buttons; p != NULL; p = p->next) {
+        Button *button = p->data;
+        gint min, nat;
+
+        gtk_widget_get_preferred_height (button->button_widget, &min, &nat);
+        button_min = MAX (button_min, min + 2 * V_PADDING);
+        button_nat = MAX (button_nat, nat + 2 * V_PADDING);
+    }
+
     if (switcher->priv->buttons_height_request > 0) {
-        requisition->height += switcher->priv->buttons_height_request;
+        *minimum += switcher->priv->buttons_height_request;
+        *natural += switcher->priv->buttons_height_request;
     } else {
-        requisition->height += button_height + V_PADDING;
+        *minimum += button_min + V_PADDING;
+        *natural += button_nat + V_PADDING;
     }
 }
 
 static void
 gdl_switcher_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
+    do_layout (GDL_SWITCHER (widget), allocation);
     gtk_widget_set_allocation (widget, allocation);
-    do_layout (GDL_SWITCHER (widget));
 }
 
 static gint
-gdl_switcher_expose (GtkWidget *widget, GdkEventExpose *event)
+gdl_switcher_draw (GtkWidget *widget, cairo_t *cr)
 {
     GSList *p;
     GdlSwitcher *switcher = GDL_SWITCHER (widget);
     if (switcher->priv->show) {
         for (p = switcher->priv->buttons; p != NULL; p = p->next) {
             GtkWidget *button = ((Button *) p->data)->button_widget;
-            gtk_container_propagate_expose (GTK_CONTAINER (widget),
-                                            button, event);
+            gtk_container_propagate_draw (GTK_CONTAINER (widget),
+                                          button, cr);
         }
     }
-    return GTK_WIDGET_CLASS (gdl_switcher_parent_class)->expose_event (widget, event);
+    return GTK_WIDGET_CLASS (gdl_switcher_parent_class)->draw (widget, cr);
 }
 
 static void
@@ -648,14 +690,12 @@ gdl_switcher_notify_cb (GObject *g_object, GParamSpec *pspec,
 }
 
 static void
-gdl_switcher_switch_page_cb (GtkNotebook *nb, GtkNotebookPage *page,
+gdl_switcher_switch_page_cb (GtkNotebook *nb, GtkWidget *page_widget,
                              gint page_num, GdlSwitcher *switcher)
 {
-    GtkWidget       *page_widget;
     gint             switcher_id;
     
     /* Change switcher button */
-    page_widget = gtk_notebook_get_nth_page (nb, page_num);
     switcher_id = gdl_switcher_get_page_id (page_widget);
     gdl_switcher_select_button (GDL_SWITCHER (switcher), switcher_id);
 }
@@ -714,9 +754,10 @@ gdl_switcher_class_init (GdlSwitcherClass *klass)
     container_class->forall = gdl_switcher_forall;
     container_class->remove = gdl_switcher_remove;
 
-    widget_class->size_request = gdl_switcher_size_request;
+    widget_class->get_preferred_width = gdl_switcher_get_preferred_width;
+    widget_class->get_preferred_height = gdl_switcher_get_preferred_height;
     widget_class->size_allocate = gdl_switcher_size_allocate;
-    widget_class->expose_event = gdl_switcher_expose;
+    widget_class->draw = gdl_switcher_draw;
     widget_class->map = gdl_switcher_map;
     
     object_class->dispose  = gdl_switcher_dispose;
